@@ -1,11 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+import warnings
 from dataclasses import dataclass, field
 
 import numpy as np
 from astropy.nddata import NDData
+from astropy import units as u
 
-from specreduce.extract import _ap_weight_image
+from specreduce.extract import _ap_weight_image, _to_spectrum1d_pixels
 from specreduce.tracing import Trace, FlatTrace
 
 __all__ = ['Background']
@@ -84,9 +86,26 @@ class Background:
                     raise ValueError('trace_object.trace_pos must be >= 1')
             return trace
 
+        if self.width < 0:
+            raise ValueError("width must be positive")
+
+        if self.width == 0:
+            self.bkg_array = np.zeros(self.image.shape[self.disp_axis])
+            return
+
         bkg_wimage = np.zeros_like(self.image, dtype=np.float64)
         for trace in self.traces:
             trace = _to_trace(trace)
+            windows_max = trace.trace.data.max() + self.width/2
+            windows_min = trace.trace.data.min() - self.width/2
+            if windows_max >= self.image.shape[self.crossdisp_axis]:
+                warnings.warn("background window extends beyond image boundaries " +
+                              f"({windows_max} >= {self.image.shape[self.crossdisp_axis]})")
+            if windows_min < 0:
+                warnings.warn("background window extends beyond image boundaries " +
+                              f"({windows_min} < 0)")
+
+            # pass trace.trace.data to ignore any mask on the trace
             bkg_wimage += _ap_weight_image(trace,
                                            self.width,
                                            self.disp_axis,
@@ -95,18 +114,22 @@ class Background:
 
         if np.any(bkg_wimage > 1):
             raise ValueError("background regions overlapped")
+        if np.any(np.sum(bkg_wimage, axis=self.crossdisp_axis) == 0):
+            raise ValueError("background window does not remain in bounds across entire dispersion axis")  # noqa
 
         if self.statistic == 'median':
             # make it clear in the expose image that partial pixels are fully-weighted
             bkg_wimage[bkg_wimage > 0] = 1
 
         self.bkg_wimage = bkg_wimage
+
         if self.statistic == 'average':
-            self.bkg_array = np.average(self.image, weights=self.bkg_wimage, axis=0)
+            self.bkg_array = np.average(self.image, weights=self.bkg_wimage,
+                                        axis=self.crossdisp_axis)
         elif self.statistic == 'median':
             med_image = self.image.copy()
             med_image[np.where(self.bkg_wimage) == 0] = np.nan
-            self.bkg_array = np.nanmedian(med_image, axis=0)
+            self.bkg_array = np.nanmedian(med_image, axis=self.crossdisp_axis)
         else:
             raise ValueError("statistic must be 'average' or 'median'")
 
@@ -185,8 +208,8 @@ class Background:
         Parameters
         ----------
         image : nddata-compatible image or None
-            image with 2-D spectral image data.  If None, will use ``image`` passed
-            to extract the background.
+            image with 2-D spectral image data.  If None, will extract
+            the background from ``image`` used to initialize the class.
 
         Returns
         -------
@@ -197,6 +220,28 @@ class Background:
 
         return np.tile(self.bkg_array, (image.shape[0], 1))
 
+    def bkg_spectrum(self, image=None):
+        """
+        Expose the 1D spectrum of the background.
+
+        Parameters
+        ----------
+        image : nddata-compatible image or None
+            image with 2-D spectral image data.  If None, will extract
+            the background from ``image`` used to initialize the class.
+
+        Returns
+        -------
+        spec : `~specutils.Spectrum1D`
+            The background 1-D spectrum, with flux expressed in the same
+            units as the input image (or u.DN if none were provided) and
+            the spectral axis expressed in pixel units.
+        """
+        bkg_image = self.bkg_image(image=image)
+
+        ext1d = np.sum(bkg_image, axis=self.crossdisp_axis)
+        return _to_spectrum1d_pixels(ext1d * getattr(image, 'unit', u.DN))
+
     def sub_image(self, image=None):
         """
         Subtract the computed background from ``image``.
@@ -204,8 +249,8 @@ class Background:
         Parameters
         ----------
         image : nddata-compatible image or None
-            image with 2-D spectral image data.  If None, will use ``image`` passed
-            to extract the background.
+            image with 2-D spectral image data.  If None, will extract
+            the background from ``image`` used to initialize the class.
 
         Returns
         -------
@@ -220,6 +265,28 @@ class Background:
         else:
             return image - self.bkg_image(image)
 
+    def sub_spectrum(self, image=None):
+        """
+        Expose the 1D spectrum of the background-subtracted image.
+
+        Parameters
+        ----------
+        image : nddata-compatible image or None
+            image with 2-D spectral image data.  If None, will extract
+            the background from ``image`` used to initialize the class.
+
+        Returns
+        -------
+        spec : `~specutils.Spectrum1D`
+            The background 1-D spectrum, with flux expressed in the same
+            units as the input image (or u.DN if none were provided) and
+            the spectral axis expressed in pixel units.
+        """
+        sub_image = self.sub_image(image=image)
+
+        ext1d = np.sum(sub_image, axis=self.crossdisp_axis)
+        return _to_spectrum1d_pixels(ext1d * getattr(image, 'unit', u.DN))
+
     def __rsub__(self, image):
         """
         Subtract the background from an image.
