@@ -19,24 +19,61 @@ __all__ = ['BoxcarExtract', 'HorneExtract', 'OptimalExtract']
 def _get_boxcar_weights(center, hwidth, npix):
     """
     Compute weights given an aperture center, half width,
-    and number of pixels
+    and number of pixels.
+
+    Based on `get_boxcar_weights()` from a JDAT Notebook by Karl Gordon:
+    https://github.com/spacetelescope/jdat_notebooks/blob/main/notebooks/MIRI_LRS_spectral_extraction/miri_lrs_spectral_extraction.ipynb
+
+    Parameters
+    ----------
+    center : float, required
+        The index of the aperture's center pixel on the larger image's
+        cross-dispersion axis.
+
+    hwidth : float, required
+        Half of the aperture's width in the cross-dispersion direction.
+
+    npix : float, required
+        The number of pixels in the larger image's cross-dispersion
+        axis.
+
+    Returns
+    -------
+    weights : `~numpy.ndarray`
+        A 2D image with weights assigned to pixels that fall within the
+        defined aperture.
     """
     weights = np.zeros((npix))
+    if hwidth == 0:
+        # the logic below would return all zeros anyways, so might as well save the time
+        # (negative widths should be avoided by earlier logic!)
+        return weights
 
-    # pixels with full weight
-    fullpixels = [max(0, int(center - hwidth + 1)),
-                  min(int(center + hwidth), npix)]
-    weights[fullpixels[0]:fullpixels[1]] = 1.0
+    if center-hwidth > npix-0.5 or center+hwidth < -0.5:
+        # entire window is out-of-bounds
+        return weights
 
-    # pixels at the edges of the boxcar with partial weight
-    if fullpixels[0] > 0:
-        w = hwidth - (center - fullpixels[0] + 0.5)
-        if w >= 0:
-            weights[fullpixels[0] - 1] = w
-        else:
-            weights[fullpixels[0]] = 1. + w
-    if fullpixels[1] < npix:
-        weights[fullpixels[1]] = hwidth - (fullpixels[1] - center - 0.5)
+    lower_edge = max(-0.5, center-hwidth)  # where -0.5 is lower bound of the image
+    upper_edge = min(center+hwidth, npix-0.5)  # where npix-0.5 is upper bound of the image
+
+    # let's avoid recomputing the round repeatedly
+    int_round_lower_edge = int(round(lower_edge))
+    int_round_upper_edge = int(round(upper_edge))
+
+    # inner pixels that get full weight
+    # the round in conjunction with the +1 handles the half-pixel "offset",
+    # the upper bound doesn't have the +1 because array slicing is inclusive on the lower index and
+    # exclusive on the upper-index
+    # NOTE: round(-0.5) == 0, which is helpful here for the case where lower_edge == -0.5
+    weights[int_round_lower_edge+1:int_round_upper_edge] = 1
+
+    # handle edge pixels (for cases where an edge pixel is fully-weighted, this will set it again,
+    # but should still compute a weight of 1.  By using N:N+1, we avoid index errors if the edge
+    # is outside the image bounds.  But we do need to avoid negative indices which would count
+    # from the end of the array.
+    if int_round_lower_edge >= 0:
+        weights[int_round_lower_edge:int_round_lower_edge+1] = round(lower_edge) + 0.5 - lower_edge
+    weights[int_round_upper_edge:int_round_upper_edge+1] = upper_edge - (round(upper_edge) - 0.5)
 
     return weights
 
@@ -46,23 +83,26 @@ def _ap_weight_image(trace, width, disp_axis, crossdisp_axis, image_shape):
     """
     Create a weight image that defines the desired extraction aperture.
 
+    Based on `ap_weight_images()` from a JDAT Notebook by Karl Gordon:
+    https://github.com/spacetelescope/jdat_notebooks/blob/main/notebooks/MIRI_LRS_spectral_extraction/miri_lrs_spectral_extraction.ipynb
+
     Parameters
     ----------
-    trace : Trace
+    trace : `~specreduce.tracing.Trace`, required
         trace object
-    width : float
+    width : float, required
         width of extraction aperture in pixels
-    disp_axis : int
+    disp_axis : int, required
         dispersion axis
-    crossdisp_axis : int
+    crossdisp_axis : int, required
         cross-dispersion axis
-    image_shape : tuple with 2 elements
+    image_shape : tuple with 2 elements, required
         size (shape) of image
 
     Returns
     -------
-    wimage : 2D image
-        weight image defining the aperture
+    wimage : `~numpy.ndarray`
+        a 2D weight image defining the aperture
     """
     wimage = np.zeros(image_shape)
     hwidth = 0.5 * width
@@ -71,11 +111,18 @@ def _ap_weight_image(trace, width, disp_axis, crossdisp_axis, image_shape):
     # loop in dispersion direction and compute weights.
     for i in range(image_shape[disp_axis]):
         # TODO trace must handle transposed data (disp_axis == 0)
-        wimage[:, i] = _get_boxcar_weights(trace[i], hwidth, image_sizes)
+        # pass trace.trace.data[i] to avoid any mask if part of the regions is out-of-bounds
+        wimage[:, i] = _get_boxcar_weights(trace.trace.data[i], hwidth, image_sizes)
 
     return wimage
 
 
+def _to_spectrum1d_pixels(fluxes):
+    # TODO: add wavelength units, uncertainty and mask to spectrum1D object
+    return Spectrum1D(spectral_axis=np.arange(len(fluxes)) * u.pixel,
+                      flux=fluxes)
+
+
 @dataclass
 class BoxcarExtract(SpecreduceOperation):
     """
@@ -153,6 +200,9 @@ class BoxcarExtract(SpecreduceOperation):
             if trace_object.trace_pos < 1:
                 raise ValueError('trace_object.trace_pos must be >= 1')
 
+        if width < 0:
+            raise ValueError("width must be positive")
+
         # weight image to use for extraction
         wimage = _ap_weight_image(
             trace_object,
@@ -163,12 +213,7 @@ class BoxcarExtract(SpecreduceOperation):
 
         # extract
         ext1d = np.sum(image * wimage, axis=crossdisp_axis)
-
-        # TODO: add wavelenght units, uncertainty and mask to spectrum1D object
-        spec = Spectrum1D(spectral_axis=np.arange(len(ext1d)) * u.pixel,
-                          flux=ext1d * getattr(image, 'unit', u.DN))
-
-        return spec
+        return _to_spectrum1d_pixels(ext1d * getattr(image, 'unit', u.DN))
 
 
 @dataclass
@@ -183,7 +228,7 @@ class HorneExtract(SpecreduceOperation):
     image : `~astropy.nddata.NDData` or array-like, required
         The input 2D spectrum from which to extract a source. An
         NDData object must specify uncertainty and a mask. An array
-        requires use of the `variance`, `mask`, & `unit` arguments.
+        requires use of the ``variance``, ``mask``, & ``unit`` arguments.
 
     trace_object : `~specreduce.tracing.Trace`, required
         The associated 1D trace object created for the 2D image.
@@ -199,22 +244,22 @@ class HorneExtract(SpecreduceOperation):
         [default: models.Polynomial1D(2)]
 
     variance : `~numpy.ndarray`, optional
-        (Only used if `image` is not an NDData object.)
+        (Only used if ``image`` is not an NDData object.)
         The associated variances for each pixel in the image. Must
-        have the same dimensions as `image`. If all zeros, the variance
+        have the same dimensions as ``image``. If all zeros, the variance
         will be ignored and treated as all ones.  If any zeros, those
         elements will be excluded via masking.  If any negative values,
         an error will be raised. [default: None]
 
     mask : `~numpy.ndarray`, optional
-        (Only used if `image` is not an NDData object.)
+        (Only used if ``image`` is not an NDData object.)
         Whether to mask each pixel in the image. Must have the same
-        dimensions as `image`. If blank, all non-NaN pixels are
+        dimensions as ``image``. If blank, all non-NaN pixels are
         unmasked. [default: None]
 
-    unit : `~astropy.units.core.Unit` or str, optional
-        (Only used if `image` is not an NDData object.)
-        The associated unit for the data in `image`. If blank,
+    unit : `~astropy.units.Unit` or str, optional
+        (Only used if ``image`` is not an NDData object.)
+        The associated unit for the data in ``image``. If blank,
         fluxes are interpreted as unitless. [default: None]
 
     """
@@ -246,7 +291,7 @@ class HorneExtract(SpecreduceOperation):
         image : `~astropy.nddata.NDData` or array-like, required
             The input 2D spectrum from which to extract a source. An
             NDData object must specify uncertainty and a mask. An array
-            requires use of the `variance`, `mask`, & `unit` arguments.
+            requires use of the ``variance``, ``mask``, & ``unit`` arguments.
 
         trace_object : `~specreduce.tracing.Trace`, required
             The associated 1D trace object created for the 2D image.
@@ -261,22 +306,22 @@ class HorneExtract(SpecreduceOperation):
             A model for the image's background flux.
 
         variance : `~numpy.ndarray`, optional
-            (Only used if `image` is not an NDData object.)
+            (Only used if ``image`` is not an NDData object.)
             The associated variances for each pixel in the image. Must
-            have the same dimensions as `image`. If all zeros, the variance
+            have the same dimensions as ``image``. If all zeros, the variance
             will be ignored and treated as all ones.  If any zeros, those
             elements will be excluded via masking.  If any negative values,
             an error will be raised.
 
         mask : `~numpy.ndarray`, optional
-            (Only used if `image` is not an NDData object.)
+            (Only used if ``image`` is not an NDData object.)
             Whether to mask each pixel in the image. Must have the same
-            dimensions as `image`. If blank, all non-NaN pixels are
+            dimensions as ``image``. If blank, all non-NaN pixels are
             unmasked.
 
-        unit : `~astropy.units.core.Unit` or str, optional
-            (Only used if `image` is not an NDData object.)
-            The associated unit for the data in `image`. If blank,
+        unit : `~astropy.units.Unit` or str, optional
+            (Only used if ``image`` is not an NDData object.)
+            The associated unit for the data in ``image``. If blank,
             fluxes are interpreted as unitless.
 
 
@@ -406,10 +451,7 @@ class HorneExtract(SpecreduceOperation):
         extraction = result * norms
 
         # convert the extraction to a Spectrum1D object
-        pixels = np.arange(img.shape[disp_axis]) * u.pix
-        spec_1d = Spectrum1D(spectral_axis=pixels, flux=extraction * unit)
-
-        return spec_1d
+        return _to_spectrum1d_pixels(extraction * unit)
 
 
 @dataclass
